<?php
/**
 * MAGECONNECT
 * Connecting TYPO3 and Magento
 * 
 * Soap Call Class
 * 
 * This class fetches user details from
 * TYPO3 and Magento
 * 
 * @author Bastian 'Aggy' Zagar
 * @version 0.1
 *
 */

require_once(t3lib_extMgm::extPath('mageconnect') . 'lib/class.tx_mageconnect_helper.php');
 
//ini_set('default_socket_timeout', 9999);
//ini_set("soap.wsdl_cache_enabled", "0");

class tx_mageconnect_soap
{
	/**
	 * Api Settings
	 * @var string
	 */
	private $_apiUrl;
	private $_apiUser;
	private $_apiKey;
		
	/**
	 * Proxy
	 */
	private $_proxy;
	
	/**
	 * Session ID
	 */
	private $_sessionId;
	
	/**
	 * Helper
	 */
	private $_helper;
	
	
	function __construct($url, $user, $key)
	{
		$this->_helper = t3lib_div::makeInstance('tx_mageconnect_helper'); 
		
		
		if (!$url || !$user || !$key) {
			return false;
		}
			
		$this->_apiUrl 	= $url . 'api/v2_soap?wsdl=1';
		$this->_apiUser = $user;
		$this->_apiKey 	= $key;	
		
	    // Connect to API
    	if ($this->connect() == FALSE) {
    		echo $this->_helper->displayError( 'Error connecting to API' );
    	}
		
	}
	
	/**
	 * _reset
	 * Resets all data
	 */
	function _reset()
	{
		$this->_apiUrl 		= NULL;
		$this->_apiUser 	= NULL;
		$this->_apiKey 		= NULL;
		
		$this->_proxy 		= NULL;
		$this->_sessionId 	= NULL;
	}
	
	/**
	 * __destruct
	 * Destructor
	 */
	function __destruct()
	{
		$this->_reset();
		unset($this->_apiUrl);
		unset($this->_apiUser);
		unset($this->_apiKey);
		
		unset($this->_proxy);
		unset($this->_sessionId);
	}
	
	/**
	 * connect
	 * Connect to the API
	 * 
	 * @return bool
	 */
	private function connect()
	{
		$params = array(
			'trace' => true,
			'connection_timeout' => 999
		);
		
		$this->_proxy		= new soapclient($this->_apiUrl, $params);
		$this->_sessionId	= $this->_proxy->login($this->_apiUser, $this->_apiKey);
		
		if ($this->_sessionId) {
			return true;
		}
		return false;
	}
	
	/**
	 * getCustomerByEmail
	 * Gets customer information by an
	 * given email address
	 * 
	 * @param string $email
	 * @return stdClass
	 */	
	public function getCustomerByEmail($email)
	{
		try {
			$customer =  $this->_proxy->customerCustomerFetch(	$this->_sessionId, 
																$email
														     );
	
			if (property_exists($customer, 'attributes')) {
				return $customer;
			}				
		} catch (Exception $e) {
			echo $this->_helper->displayError( $e->getMessage() );
		}
		return;
	}
	
	/**
	 * getProductsList
	 * Get a list of existing products
	 * 
	 * @return stdClass
	 */
	public function getProductsList()
	{
		try {
			$products =  $this->_proxy->catalogProductList(	$this->_sessionId, 
															'',		// Filters
															''		// StoreView
														  );
															 
			if (count($products) > 0) {
				return $products;
			}				
		} catch (Exception $e) {
			echo $this->_helper->displayError( $e->getMessage() );
		}
		return;
	}
	
	/**
	 * getProductById
	 * Get product information by an given product id
	 * 
	 * @param int $productId
	 * @return stdClass
	 */
	public function getProductById($productId)
	{
		try {
			$product =  $this->_proxy->catalogProductDetail(	$this->_sessionId, 
																$productId,		// Product
																'',				// StoreView
																''				// Product Identifier Type
														  );
													  
			if ($product) {
				return unserialize($product);
			}											  
															 
		} catch (Exception $e) {
			echo $this->_helper->displayError( $e->getMessage() );
		}
		return;
	}
	
	
}

if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/mageconnect/lib/class.tx_mageconnect_soap.php'])	{
	include_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/mageconnect/lib/class.tx_mageconnect_soap.php']);
}
