<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2011 Bastian Zagar (magedeveloper.de) <kontakt@magedeveloper.de>
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/
/**
 * [CLASS/FUNCTION INDEX of SCRIPT]
 */
require_once(PATH_tslib.'class.tslib_pibase.php');

require_once(t3lib_extMgm::extPath('mageconnect') . 'lib/class.tx_mageconnect_product.php');
require_once(t3lib_extMgm::extPath('mageconnect') . 'lib/class.tx_mageconnect_helper.php');
require_once(t3lib_extMgm::extPath('mageconnect') . 'lib/class.tx_mageconnect_cache.php');

/**
 * Plugin 'MageConnect Login Form' for the 'mageconnect' extension.
 *
 * @author	Bastian Zagar (magedeveloper.de) <kontakt@magedeveloper.de>
 * @package	TYPO3
 * @subpackage tx_mageconnect
 */
class tx_mageconnect_pi2 extends tslib_pibase {
	
	var $prefixId      = 'tx_mageconnect_pi2';		// Same as class name
	var $scriptRelPath = 'pi2/class.tx_mageconnect_pi2.php';	// Path to this script relative to the extension dir.
	var $extKey        = 'mageconnect';	// The extension key.
	//CACHING var $pi_checkCHash = true;
	
	var $_helper;
	var $_products;
	var $_pid;
	var $conf;
	
	function __construct()
	{
		$this->_helper 		= t3lib_div::makeInstance('tx_mageconnect_helper'); 
	}
	
	/**
	 * _getFlexformConfig
	 * Get the flexform config by typename and fieldname
	 * 
	 * @param string $type
	 * @param string $field
	 * @return string
	 */
	function _getFlexformConfig($field, $type)
	{
		return $this->pi_getFFValue($this->cObj->data['pi_flexform'], $field, $type); 
	}	
	
	/**
	 * getProductDisplayType
	 * Determine the product display type that
	 * was made in the flexform config of the
	 * plugin
	 * 
	 * @return string
	 */
	function getProductDisplayType()
	{
		return $this->_getFlexformConfig('display_type', 'product_setting');
	}
	
	/**
	 * _getConfig
	 * Return config data from a given
	 * key
	 * 
	 * @param string $key
	 * @return string
	 */
	function _getConfig($key)
	{
		return $this->conf[$key];
	}
	
	/**
	 * getReloadConfig
	 * Get the reload configuration setting
	 * 
	 * @return bool
	 */
	function getReloadConfig()
	{
		$config = $this->_getFlexformConfig('always_load_from_soap', 'general_setting');
		
		if ($config === '1') {
			return true;
		} 
		return false;
	}
	
	/**
	 * getSelectedProducts
	 * Determine the products ids that have been
	 * set in the backend
	 * 
	 * @return array
	 */
	function getFlexformSelectedProducts()
	{
		$products = $this->_getFlexformConfig('product_single', 'product_setting');
		$products = t3lib_div::trimExplode(',',$products,1);
		
		return $products;
	}
	
	/**
	 * getSelectedProducts
	 * Gets the selected products and imports
	 * the ones, that are needed or forced by reload config
	 * 
	 * @return array
	 */
	function getSelectedProducts()
	{
		$selectedProducts = $this->getFlexformSelectedProducts();
		
		// If we don't need to reload all products, we want only new ones!
		if (!$this->getReloadConfig()) {
				
			// Check if Product Data already exists in database
			$existingProducts = $this->_products->getExistingProducts();
			
			$productsToFetch = $selectedProducts;
			foreach ($existingProducts as $_ex) {
				foreach ($productsToFetch as $k=>$_s) {
					if ($_ex === $_s) {
						unset($productsToFetch[$k]);
					}
				}
			}
			
			if (count($productsToFetch) > 0) {
				// If products don't exist in database, we need to fetch them by soap
				$result = $this->_products->importProducts($productsToFetch);			
			}
			
		} else {
			// Import all products
			$result = $this->_products->importProducts($selectedProducts);	
		}
		return $selectedProducts;
	}
	
	
	/******************************************
	 * DISPLAY FUNCTIONS
	 *****************************************/
	
	/**
	 * displayProductList
	 * Displays a product list
	 * 
	 * @return string
	 */
	function displayProductList()
	{
		$selectedProducts = $this->getSelectedProducts();
		$products = $this->_products->getProductsFromDb($selectedProducts);
		
		// Load Template File
		$templateHtml 		= $this->cObj->fileResource( $this->_getConfig('templateProductList') );
		$productListHtml 	= $this->cObj->getSubpart($templateHtml, '###PRODUCT_LIST###');
		$productHtml		= $this->cObj->getSubpart($productListHtml, '###PRODUCT_ITEM###');
		
		$subPartContent = ''; $item = 0; $items = count($products); $even = true; $line = '';
		foreach ($selectedProducts as $_id) {
			foreach ($products as $_product) {
				
				if ($_id === $_product['uid']) {
					$markers = $this->_products->getProductMarkers($_product);

					// Even/Odd CSS Class Determination
					if ($even === true) {
						$line = 'even';
						$even = false;
					} else {
						$line = 'odd';
						$even = true;
					}
					
					// Class Determination First/Last
					if ($item == 0) {
						$markers['###CLASS###'] = 'class=\'first '.$line.'\''; 
					} else if ($item === $items) {
						$markers['###CLASS###'] = 'class=\'last '.$line.'\''; 
					} else {
						$markers['###CLASS###'] = 'class=\''.$line.'\''; 
					}
					
					// Check if the product has an image
					$imageHtml = '<p>'.$this->pi_getLL('template.label_no_image_available').'</p>';
					if ($markers['###DETAIL_IMAGE###'] != 'no_selection') {
						$imageHtml = $this->cObj->getSubpart($productHtml, '###PRODUCT_IMAGE###');
					}
					
					$p = $this->cObj->substituteSubpart($productHtml, '###PRODUCT_IMAGE###', $imageHtml);
					$subPartContent .= $this->cObj->substituteMarkerArrayCached($p, $markers);
					$item++;
				}
			}
		}
		return $this->cObj->substituteSubpart($productListHtml, '###PRODUCT_ITEM###', $subPartContent);
	}
	
	/**
	 * displayProductGrid
	 * Displays a product grid
	 * 
	 * @return string
	 */
	function displayProductGrid()
	{
		$selectedProducts = $this->getSelectedProducts();
		$products = $this->_products->getProductsFromDb($selectedProducts);
		
		// Load Template File
		$templateHtml 		= $this->cObj->fileResource( $this->_getConfig('templateProductGrid') );
		$productListHtml 	= $this->cObj->getSubpart($templateHtml, '###PRODUCT_LIST###');
		$productHtml		= $this->cObj->getSubpart($productListHtml, '###PRODUCT_ITEM###');
		
		$subPartContent = '';
		foreach ($selectedProducts as $_id) {
			foreach ($products as $_product) {
				
				if ($_id === $_product['uid']) {
					$markers = $this->_products->getProductMarkers($_product);
					
					// Check if the product has an image
					$imageHtml = '<p>'.$this->pi_getLL('template.label_no_image_available').'</p>';
					if ($markers['###DETAIL_IMAGE###'] != 'no_selection') {
						$imageHtml = $this->cObj->getSubpart($productHtml, '###PRODUCT_IMAGE###');
					}
					
					$p = $this->cObj->substituteSubpart($productHtml, '###PRODUCT_IMAGE###', $imageHtml);
					$subPartContent .= $this->cObj->substituteMarkerArrayCached($p, $markers);
				}
			}
		}
		return $this->cObj->substituteSubpart($productListHtml, '###PRODUCT_ITEM###', $subPartContent);
	}
	
	/**
	 * displayProductSingle
	 * Displays a small box with
	 * product information
	 * 
	 * @return string
	 */
	function displayProductSingle()
	{
		$selectedProducts = $this->getSelectedProducts();
		$products = $this->_products->getProductsFromDb($selectedProducts);
		
		// Load Template File
		$templateHtml 		= $this->cObj->fileResource( $this->_getConfig('templateProductSingle') );
		$productListHtml 	= $this->cObj->getSubpart($templateHtml, '###PRODUCT_LIST###');
		$productHtml		= $this->cObj->getSubpart($productListHtml, '###PRODUCT_ITEM###');
		
		$subPartContent = '';
		foreach ($selectedProducts as $_id) {
			foreach ($products as $_product) {
				
				if ($_id === $_product['uid']) {
					$markers = $this->_products->getProductMarkers($_product);
					
					// Check if the product has an image
					$imageHtml = '<p>'.$this->pi_getLL('template.label_no_image_available').'</p>';
					if ($markers['###DETAIL_IMAGE###'] != 'no_selection') {
						$imageHtml = $this->cObj->getSubpart($productHtml, '###PRODUCT_IMAGE###');
					}
					
					$p = $this->cObj->substituteSubpart($productHtml, '###PRODUCT_IMAGE###', $imageHtml);
					$subPartContent .= $this->cObj->substituteMarkerArrayCached($p, $markers);
				}
			}
		}
		return $this->cObj->substituteSubpart($productListHtml, '###PRODUCT_ITEM###', $subPartContent);
	}
	
	/**
	 * displayProductDetail
	 * Display product detail information
	 * 
	 * @return string
	 */
	function displayProductDetail()
	{
		$selectedProducts = $this->getSelectedProducts();
		$products = $this->_products->getProductsFromDb($selectedProducts);
		
		// Load Template File
		$templateHtml 		= $this->cObj->fileResource( $this->_getConfig('templateProductDetail') );
		$detailHtml  		= $this->cObj->getSubpart($templateHtml, '###PRODUCT_DETAIL###');
		$productHtml 		= $this->cObj->getSubpart($templateHtml, '###PRODUCT_ITEM###');
		$attributeHtml		= $this->cObj->getSubpart($templateHtml, '###PRODUCT_ATTRIBUTES###');
		$attributePart		= $this->cObj->getSubpart($templateHtml, '###ATTRIBUTES###');
		
		$subPartContent = '';
		foreach ($selectedProducts as $_id) {
			foreach ($products as $_product) {
				
				if ($_id === $_product['uid']) {
					$markers = $this->_products->getProductMarkers($_product);
					
					$attributes = unserialize($_product['attributes']);
					$attributes = $attributes['additional_attributes'];
					
					// Create Attribute Table
					$attributeSubPart = ''; $attributeMarkers = array(); $attributeInfo = '';
					if (count($attributes) > 0) {
						foreach ($attributes as $_attribute) {
							$attributeMarkers['###ATTRIBUTE_NAME###'] = $_attribute['label'];
							$attributeMarkers['###ATTRIBUTE_VALUE###'] = $_attribute['value'];
							$attributeSubPart .= $this->cObj->substituteMarkerArrayCached($attributePart, $attributeMarkers);
						}
						// Fill final attribute table with single rows
						$attributeInfo = $this->cObj->substituteSubpart($attributeHtml, '###ATTRIBUTES###', $attributeSubPart);
					}

					
					// Check if the product has an image
					$imageHtml = '<p>'.$this->pi_getLL('template.label_no_image_available').'</p>';
					if ($markers['###DETAIL_IMAGE###'] != 'no_selection') {
						$imageHtml = $this->cObj->getSubpart($productHtml, '###PRODUCT_IMAGE###');
					}
					// Insert Image to Template
					$p = $this->cObj->substituteSubpart($productHtml, '###PRODUCT_IMAGE###', $imageHtml);
					
					// Insert Finished Attribute Table to Template
					$p = $this->cObj->substituteSubpart($p, '###PRODUCT_ATTRIBUTES###', $attributeInfo);
					
					$subPartContent .= $this->cObj->substituteMarkerArrayCached($p, $markers);
				}
			}
		}
		return $this->cObj->substituteSubpart($detailHtml, '###PRODUCT_ITEM###', $subPartContent);
	}
	
	/**
	 * displayProductMarkers
	 * Display product available template markers
	 * 
	 * @return string
	 */
	function displayProductMarkers()
	{
		$selectedProducts = $this->getSelectedProducts();
		$products = $this->_products->getProductsFromDb($selectedProducts);
		
		// Load Template File
		$templateHtml 		= $this->cObj->fileResource( $this->_getConfig('templateProductMarkers') );
		$markerListHtml		= $this->cObj->getSubpart($templateHtml, '###MARKER_LIST###');
		$productListHtml	= $this->cObj->getSubpart($markerListHtml, '###PRODUCT_LIST###');
		$markerDetailHtml	= $this->cObj->getSubpart($productListHtml, '###MARKER_DETAIL###');
		
		$subPartContent = '';
		foreach ($selectedProducts as $_id) {
			foreach ($products as $_product) {	
				
				if ($_id === $_product['product_id']) {
					
					$markers = $this->_products->getProductMarkers($_product);
					$markerSubContent = '';
					foreach ($markers as $_marker=>$_value) {
						$markerArr = array();
						$markerArr['###MARKER###'] = $_marker;
						$markerArr['###VALUE###'] = htmlentities( $_value );
						$markerSubContent .= $this->cObj->substituteMarkerArrayCached($markerDetailHtml, $markerArr);
					}
					
					$subPartContent .= $this->cObj->substituteSubpart($productListHtml, '###MARKER_DETAIL###', $markerSubContent);
				}	
			}
		}
		return $subPartContent;
	}
	/*****************************************/	
	
	
	
		
	/**
	 * The main method of the PlugIn
	 *
	 * @param	string		$content: The PlugIn content
	 * @param	array		$conf: The PlugIn configuration
	 * @return	The content that is displayed on the website
	 */
	function main($content, $conf) 
	{
		
		$this->pi_initPIflexForm();    // Initialize the FlexForms array 
		
		$this->conf = $conf;
		$this->pi_setPiVarDefaults();
		$this->pi_loadLL();
		$this->pi_USER_INT_obj = 0;
		
		// Initialize Products Class
		$this->_products = t3lib_div::makeInstance('tx_mageconnect_product');
		$this->_products->init($this->cObj, $conf);

		// Determine display type		
		$displayType = $this->getProductDisplayType();
		
		switch ($displayType) {
			
				// Products list
				case 'display_list':
					$content .= $this->displayProductList();
					break;
				// Products grid
				case 'display_grid':
					$content .= $this->displayProductGrid();
					break;	
				// Single product
				case 'display_single':
					$content .= $this->displayProductSingle();
					break;	
				// Product details
				case 'display_detail':
					$content .= $this->displayProductDetail();
					break;	
				// Product markers
				case 'display_markers':
					$content .= $this->displayProductMarkers();
					break;	
				default:
					$content .= '';
					break;	
				
		}
		
		return $this->pi_wrapInBaseClass($content);
	}
}

if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/mageconnect/pi2/class.tx_mageconnect_pi2.php'])	{
	include_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/mageconnect/pi2/class.tx_mageconnect_pi2.php']);
}


?>